package itemhandler;

import java.io.Serializable;
import java.security.InvalidParameterException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import utility.Parameter;
/**
 *Item class has 10 parameters constructed in constructor, Item has getters and setters
 * for all fields.
 *
 * @author Lucas
 *
 * @version 27.10.2022
 */

public class Item implements Serializable
{
    private String number;
    private static final String UNDEFINED_OPTIONAL_STRING = "not defined";
    private String description = UNDEFINED_OPTIONAL_STRING;
    private String color = UNDEFINED_OPTIONAL_STRING;
    private String brand = UNDEFINED_OPTIONAL_STRING;
    private int price;
    private int category;
    private int quantity;
    private double weight;
    private double length;
    private double height;


    /**
     * Constructs an item of class itemPackage.Item that contains the items properties.
     *
     * @param number           itemNumber is used to identify item, contains numbers and letters
     * @param price            The price of the item
     * @param category         Number to represent item
     *                             category 1: Floor laminates, 2: Windows, 3: Doors, 4: Timber
     * @param quantity The number of item currently in stock
     * @param weight           The weight of the item
     * @param length           The length of the item
     * @param height           The height of the item
     */
    public Item(String number, int price,
                int category, int quantity, double weight,
                double length, double height) throws InvalidParameterException
    {
        setPrice(price);
        setCategory(category);
        setQuantity(quantity);
        setWeight(weight);
        setLength(length);
        setHeight(height);
        setNumber(number);
    }

    /**
     * Empty item constructor.
     */
    public Item()
    {

    }

    /**
     * gets and returns item information mapped as a linked hashmap maintain order in the order
     * String, int, double used in JTable frame in ItemTable.
     *
     * @return itemHashMap item details mapped as a type HashMap,
     *      key is parameter name while value is parameter value
     */
    public Map<String, String> getItemInformationMapped()
    {
        HashMap<String, String> itemHashMap = new LinkedHashMap<>();
        itemHashMap.put("Number", number);
        itemHashMap.put("Description", description);
        itemHashMap.put("Brand", brand);
        itemHashMap.put("Color", color);
        itemHashMap.put("Price", String.valueOf(price));
        itemHashMap.put("Category", String.valueOf(category));
        itemHashMap.put("Quantity", String.valueOf(quantity));
        itemHashMap.put("Weight", String.valueOf(weight));
        itemHashMap.put("Length", String.valueOf(length));
        itemHashMap.put("Height", String.valueOf(height));

        return itemHashMap;
    }
    /** returns number.

     * @return number
     */
    public String getNumber()
    {
        return number;
    }

    /** gets brand.

     * @return brand
     */

    public String getBrand()
    {
        return brand;
    }

    /** getter for color.

     * @return color
     */
    public String getColor()
    {
        return color;
    }

    /** getter for description.

     * @return description
     */
    public String getDescription()
    {
        return description;
    }

    /** getter for height.

     * @return itemHeight
     */
    public double getHeight()
    {
        return height;
    }

    /** getter for length.

     * @return length
     */
    public double getLength()
    {
        return length;
    }

    /** getter for weight.

     * @return weight
     */
    public double getWeight()
    {
        return weight;
    }

    /** getter for category.

     * @return category
     */
    public int getCategory()
    {
        return category;
    }

    /** gets price.

     * @return price
     */
    public int getPrice()
    {
        return price;
    }

    /** gets quantity.

     * @return quantity
     */
    public int getQuantity()
    {
        return quantity;
    }

    /** sets brand, throws InvalidParameterException if invalid data.

     * @param brand brand
     */
    public void setBrand(String brand)
    {
        if (Parameter.checkValid(brand))
        {
            this.brand = brand;
        } else
        {
            throw new InvalidParameterException();
        }

    }

    /** sets category, throws InvalidParameterException if invalid data.

     * @param category item category
     */
    public void setCategory(int category)
    {
        if (Parameter.checkValidCategory(category))
        {
            this.category = category;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets color, throws InvalidParameterException if invalid data.

     * @param color item color
     */
    public void setColor(String color)
    {
        if (Parameter.checkValidColor(color))
        {
            this.color = color;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets description, throws InvalidParameterException if invalid data.

     * @param description item description
     */
    public void setDescription(String description)
    {
        if (Parameter.checkValid(description))
        {
            this.description = description;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets height, throws InvalidParameterException if invalid data.

     * @param height item height
     */
    public void setHeight(double height)
    {
        if (Parameter.checkValid(height))
        {
            this.height = height;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets length, throws InvalidParameterException if invalid data.

     * @param length item length
     */
    public void setLength(double length)
    {
        if (Parameter.checkValid(length))
        {
            this.length = length;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets price, throws InvalidParameterException if invalid data.

     * @param price item price
     */
    public void setPrice(int price)
    {
        if (Parameter.checkValid(price))
        {
            this.price = price;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets weight, throws InvalidParameterException if invalid data.

     * @param itemWeight item weight
     */
    public void setWeight(double itemWeight)
    {
        if (Parameter.checkValid(itemWeight))
        {
            this.weight = itemWeight;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets quantity, throws InvalidParameterException if invalid data.

     * @param quantity the number of items in stock
     */
    public void setQuantity(int quantity)
    {
        if (Parameter.checkValid(quantity))
        {
            this.quantity = quantity;
        } else
        {
            throw new InvalidParameterException();
        }
    }

    /** sets number, throws InvalidParameterException if invalid data. Is private because
     * item number is permanent.

     * @param number item number
     */
    private void setNumber(String number)
    {
        if (Parameter.checkValid(number))
        {
            this.number = number;
        } else
        {
            throw new InvalidParameterException();
        }
    }
}