package itemhandler;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * Saves and loads a map to a text file in binary format.
 * Adapted from Sean Bright <a href="https://stackoverflow.com/a/41622773">Stack Exchange</a>.
 */
public class ItemRegistryStorage
{
    private static final String PATH = "resources/ItemRegistryStorage.txt";

    /**
     * Saves map to file.

     * @param itemMap itemRegistry to be stored.
     */
    public void saveFile(Map<String, Item> itemMap) throws IOException
    {
        try (ObjectOutputStream objectOutput = new ObjectOutputStream(new FileOutputStream(PATH)))
        {
            objectOutput.writeObject(itemMap);
        }
    }

    /**
     * Reads file to store hashmap.

     * @return hashMap of type String, Item
     * @throws IOException no itms were found
     */
    public Map<String, Item> readFile() throws ClassNotFoundException, IOException
    {
        try (ObjectInputStream objectInput = new ObjectInputStream(new FileInputStream(PATH)))
        {
            // Unable to resolve warning even with the use of an instanceof if statement.
            return ((HashMap<String, Item>) objectInput.readObject());
        }
    }
}
