package staticconstants;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Enum that stores category with categoryNumber.
 */
public enum Category
{
    FLOOR_LAMINATE(1),
    WINDOW(2),
    DOOR(3),
    TIMBER(4),
    INVALID(0);
    private final int categoryNumber;

    /**
     * Constructor for category enum.

     * @param categoryNumber number that corresponds to category
     */
    Category(int categoryNumber)
    {
        this.categoryNumber = categoryNumber;
    }


    /**
     * Getter for categoryNumber.

     * @return categoryNumber
     */
    private int getCategoryNumber()
    {
        return categoryNumber;
    }

    /**
     * Gets category string that corresponds to the category number.

     * @param categoryNumber number that corresponds to category
     * @return category as string corresponding to number
     */
    public static String getCategoryFromNumber(int categoryNumber)
    {
        boolean flag = true;
        int index = 0;
        Category category = INVALID;
        while (flag && (index < Category.values().length))
        {
            category = Category.values()[index];
            if (category.getCategoryNumber() == categoryNumber)
            {
                flag = false;
            } else
            {
                index++;
            }
        }
        return category.name().toLowerCase();
    }
    /**
     * gets category mapped where category and corresponding number are mapped.

     * @return map of category and category number
     */
    public static Map<String, String> getCategoryMap()
    {
        HashMap<String, String> categoryMap = new LinkedHashMap<>();
        Category category;
        for (int index = 0; index < Category.values().length - 1; index++)
        {
            category = Category.values()[index];
            categoryMap.put(category.name(),
                String.valueOf(category.getCategoryNumber()));
        }
        return categoryMap;
    }
}
