package utility;

import itemhandler.Item;
import java.security.InvalidParameterException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import staticconstants.ItemFieldEnum;

/**
 * Class used to filter item description, brand, category and color.
 */
public class ItemFilter
{

    /**
     * Chooses which field is to be used to filter.

     * @param stringUsedToFilter String used to filter itemRegistry
     * @param itemMap map of Item
     * @param field enum of the field to be used in filter
     * @return filtered map
     */
    public Map<String, Item> filterItemRegistry(String stringUsedToFilter,
                                                Map<String, Item> itemMap, ItemFieldEnum field)
    {
        Map<String, Item> newMap;
        switch (field)
        {
            case DESCRIPTION -> newMap = filterItemDescription(stringUsedToFilter, itemMap);
            case COLOR -> newMap = filterItemColor(stringUsedToFilter, itemMap);
            case CATEGORY -> newMap = filterItemCategory(Integer.parseInt(stringUsedToFilter),
                itemMap);
            case BRAND -> newMap = filterItemBrand(stringUsedToFilter, itemMap);
            default -> throw new InvalidParameterException();
        }
        return newMap;
    }
    /**
     * Filters item in item registry to return an ArrayList containing items that contain
     * stringToUsedToFilter in the description.

     * @param stringToUsedToFilter String used to filter itemRegistry
     * @param itemMap itemRegistry
     * @return itemArrayList new ArrayList containing what was to be filtered
     */
    private Map<String, Item> filterItemDescription(String stringToUsedToFilter,
                                                   Map<String, Item> itemMap)
    {
        Iterator<Item> it = itemMap.values().iterator();
        HashMap<String, Item> sortedItemMap = new HashMap<>();
        while (it.hasNext())
        {
            Item item = it.next();
            if (item.getDescription().toLowerCase().contains(stringToUsedToFilter.toLowerCase()))
            {
                sortedItemMap.put(item.getNumber(), item);
            }
        }
        return sortedItemMap;
    }

    /**
     * Filters item in item registry to return an ArrayList containing items that contain
     * stringUsedToFilter in the description.

     * @param stringUsedToFilter String used to filter itemRegistry
     * @param itemMap itemRegistry
     * @return sortedItemMap
     */
    private Map<String, Item> filterItemColor(String stringUsedToFilter, Map<String, Item> itemMap)
    {
        Iterator<Item> itemRegistryIterator = itemMap.values().iterator();
        HashMap<String, Item> sortedItemMap = new HashMap<>();
        while (itemRegistryIterator.hasNext())
        {
            Item item = itemRegistryIterator.next();
            if (item.getColor().toLowerCase().contains(stringUsedToFilter.toLowerCase()))
            {
                sortedItemMap.put(item.getNumber(), item);
            }
        }
        return sortedItemMap;
    }

    /**
     * Filters item in item registry to return an ArrayList containing items that contain
     * stringUsedToFilter in the description.

     * @param stringUsedToFilter String used to filter itemRegistry
     * @param itemMap itemRegistry
     * @return sortedItemMap
     */
    private Map<String, Item> filterItemCategory(int stringUsedToFilter,
                                                Map<String, Item> itemMap)
    {
        Iterator<Item> itemRegistryIterator = itemMap.values().iterator();
        HashMap<String, Item> sortedItemMap = new HashMap<>();
        while (itemRegistryIterator.hasNext())
        {
            Item item = itemRegistryIterator.next();

            if (item.getCategory() == stringUsedToFilter)
            {
                sortedItemMap.put(item.getNumber(), item);
            }
        }
        return sortedItemMap;
    }

    /**
     * Filters item in item registry to return an ArrayList containing items that contain
     * stringUsedToFilter in the description.

     * @param stringUsedToFilter String used to filter itemRegistry
     * @param itemMap itemRegistry
     * @return sortedItemMap
     */
    private Map<String, Item> filterItemBrand(String stringUsedToFilter,
                                                Map<String, Item> itemMap)
    {
        Iterator<Item> itemRegistryIterator = itemMap.values().iterator();
        HashMap<String, Item> sortedItemMap = new HashMap<>();
        while (itemRegistryIterator.hasNext())
        {
            Item item = itemRegistryIterator.next();
            if (item.getBrand().toLowerCase().contains(stringUsedToFilter.toLowerCase()))
            {
                sortedItemMap.put(item.getNumber(), item);
            }
        }
        return sortedItemMap;
    }
}
