package utility;

import itemhandler.Item;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;
import staticconstants.ItemFieldEnum;

/**
 * Sort item according to item price, quantity, weight, length or height.
 */
public class ItemSorter
{

    /**
     * private constructor to hide implicit public one.
     */
    private ItemSorter()
    {

    }

    /**
     * Adds item from registry to a treemap that sorts the keys in numerical order where the key
     * is item price or quantity.

     * @param map item registry
     * @param parameter what parameter is to be used to sort
     * @return new item registry in a new order
     * @throws InvalidParameterException throws when an unexpected field enum is put in
     */
    public static Map<String, Item> sortItemInteger(Map<String, Item> map,
                                                    ItemFieldEnum parameter)
        throws InvalidParameterException
    {
        TreeMap<Integer, ArrayList<Item>> integerSortedMap = new TreeMap<>();
        Iterator<Item> itemIterator = map.values().iterator();
        Item item;
        while (itemIterator.hasNext())
        {
            item = itemIterator.next();
            switch (parameter)
            {
                case PRICE ->
                {
                    integerSortedMap.putIfAbsent(item.getPrice(), new ArrayList<>());
                    integerSortedMap.get(item.getPrice()).add(item);
                }
                case QUANTITY ->
                {
                    integerSortedMap.putIfAbsent(item.getQuantity(),
                        new ArrayList<>());
                    integerSortedMap.get(item.getQuantity()).add(item);
                }
                default -> throw new InvalidParameterException();
            }
        }
        return toStringMap(integerSortedMap.values());
    }

    /**
     * Adds item from registry to a treemap that sorts the keys in numerical order where the key
     * is item height, weight, or length.
     *
     * @param map item registry
     * @param parameter which double field is to be used to sort.
     * @return item map of type Double, Arraylist
     */
    public static Map<String, Item> sortItemDouble(Map<String, Item> map,
                                                   ItemFieldEnum parameter)
    {
        TreeMap<Double, ArrayList<Item>> doubleSortedMap = new TreeMap<>();
        Iterator<Item> itemIterator = map.values().iterator();
        Item item;
        while (itemIterator.hasNext())
        {
            item = itemIterator.next();
            switch (parameter)
            {
                case HEIGHT ->
                {
                    doubleSortedMap.putIfAbsent(item.getHeight(), new ArrayList<>());
                    doubleSortedMap.get(item.getHeight()).add(item);
                }
                case WEIGHT ->
                {
                    doubleSortedMap.putIfAbsent(item.getWeight(), new ArrayList<>());
                    doubleSortedMap.get(item.getWeight()).add(item);
                }
                case LENGTH ->
                {
                    doubleSortedMap.putIfAbsent(item.getLength(), new ArrayList<>());
                    doubleSortedMap.get(item.getLength()).add(item);
                }
                default -> throw new InvalidParameterException();
            }

        }
        return toStringMap(doubleSortedMap.values());
    }

    private static Map<String, Item> toStringMap(Collection<ArrayList<Item>> itemCollection)
    {
        Iterator<ArrayList<Item>> itemIterator = itemCollection.iterator();
        HashMap<String, Item> sortedMap = new LinkedHashMap<>();
        while (itemIterator.hasNext())
        {

            for (Item item : itemIterator.next())
            {
                sortedMap.put(item.getNumber(), item);
            }
        }
        return sortedMap;
    }

}
